"""Command-line interfaces to the library.

This package provides command-line programs using the library.  These are what
seem useful to me, so they're included with the library.  Third-party users of
the library can create their own programs.

For now, all such CLI programs are prefixed with 'hk'.
"""
